#include <stdio.h>
#include <zlib.h>
#include "config.h"
#include "mswordview.h"

extern FILE *outputfile;
extern FILE *erroroutput;

/*
written by peter.brandstrom@ericsson.com

Copyright (C) 1998
    Peter Brandstrom
*/

/*
wmf files are often stored compressed with lz compression in word files

<quote from documentation>
They are normally stored in a compressed format using the LZ compression 
algorithm in the format used by GNU Zip deflate/inflate with a 32k window. 
The format is zlib format . The only metafile compression version number 
currently defined identifies this format and is analogous to the PNG 
compression type value in the PNG file format.
</quote>

Peter Brandstrom - D/Z <peter.brandstrom@ericsson.com> took at look at it
and said
<quote>
The compressed data begins with a zlib header which seems to be two
bytes containing the compression method, window size and check byte. The
zlib default is a 32k window.
</quote>
*/

int decompress(FILE *inputfile,char *outputfile,U32 inlen,U32 outlen)
{
#ifdef SYSTEM_ZLIB
  char *compr;
  char *uncompr;
  FILE* fp;
  int err;
  uLong uncomprLen, comprLen;
  int temp;

  /*
  z_verbose = 1;
  */
  
  fp = inputfile;
  
  /* set the size of the file*/
  comprLen = inlen;

  /* Read in the file contents */
#if 0 
  compr = malloc(comprLen);
  uncompr = malloc(outlen);
  if (compr == NULL) 
  	{
	fprintf(erroroutput,"no mem to decompress wmf files\n");
	return(-1);
	}
  if (uncompr == NULL) 
  	{
	fprintf(erroroutput,"no mem to decompress wmf files\n");
	free(compr);
	return(-1);
	}
#endif
 
  temp = fread(compr, 1,comprLen, fp);
  if (comprLen != temp)
	{
	fprintf(erroroutput,"not able to read in wmf file for decompression, %d %d\n",comprLen,temp); 
	free(compr);
	free(uncompr);
	return(-1);
	}
  fclose(fp);

  uncomprLen = outlen;	/* This was the trick :( */
  err = uncompress(uncompr, &uncomprLen, compr, comprLen);

  if (err != Z_OK) 
  {
  fprintf(erroroutput, "error: %d\n", err); 
  free(compr);
  free(uncompr);
  return(-1); 
  } 

  /* Write out uncompressed data */
  fp = fopen(outputfile, "w");
  if (fp == NULL)
	{
	fprintf(erroroutput,"not able to open %s for writing\n",outputfile); 
	free(compr);
	free(uncompr);
	return(-1);
	}
  if (uncomprLen != fwrite(uncompr, uncomprLen, 1, fp))
  	{
	fprintf(erroroutput,"error writing to %s\n",outputfile); 
	free(compr);
	free(uncompr);
	return(-1);
	}
  fclose(fp);
  free(compr);
  free(uncompr);
  return 0;
#endif
}
